%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
% Computing standard errors for model parameters using a bootstrap method
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function output = bstrap_unrestricted(data,nsp,A)

% data: we use original raw data for bootstrap resampling

[n1,n2] = size(data);

n = n1/3;

unif = n*rand(nsp,n); tbeta = []; t1 = 0;

for i1 = 1:nsp
%%%%%%%% Resampling %%%%%%%%%%        
    rsam = unif(i1,:);
    
    rdt = [];

    for j = 1:n
        for k = 1:n
            if (((k-1) <= rsam(j))&(rsam(j) < k))
                m1 = 3*(k-1)+1; m2 = 3*k;
                rdt = [rdt;data(m1:m2,3:4)];    
            end
        end
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% Sorting data according to states %%%%%%%%%%%%%%%%    
    dt = rdt;

    d10 = []; d20 = []; d21 = []; d22 = []; 

    for i = 1:n
    
        m1 = 3*(i-1)+1; m2 = 3*i;
    
        round1 = dt(m1:m2,:); round = ones(3,2);
    
        for j = 1:3
            for k = 1:2    
                if (round1(j,k) == -1)
                    round(j,k) = 0;                
                end
            end
        end
    
        s2 = sum(round(:,1));

        for j = 1:3
            d10 = [d10;round(j,1)];    
        end
    
        if (s2 == 0)
            for j = 1:3
                d20 = [d20;round(j,2)];    
            end
        elseif (s2 == 1)
            for j = 1:3
                if (round(j,1) == 0)
                    d21 = [d21;round(j,2)];
                end
            end
        elseif (s2 == 2)
            for j = 1:3
                if (round(j,1) == 0)
                    d22 = [d22;round(j,2)];
                end
            end        
        end   
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    z1 = length(d22);
%%%%%%%%%%%%%%%%%% Estimation with resampled data %%%%%%%%%%%%%%
% The resampled data is used if there are samples at all states 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if (z1 >= 1)
        
        %t1 = t1 + 1
        
        options = optimset('Display','off');
        
        for i = 1:2
            if (i == 1)
                if (A == 3)
                    xbeta0 = [3.2668;0.125;-6.9118;2.3896];
                elseif (A == 1.5)
                    xbeta0 = [2.0662;0.6794;7.1858;5.8185];
                end
    
                [xbeta1,fval1] = fminsearch(@lolik,xbeta0,options,d10,d20,d21,d22,A);
            else
                if (A == 3)
                    xbeta0 = [0;0;0;0];
                elseif (A == 1.5)
                    xbeta0 = [0;0;0;0];
                end
    
                [xbeta2,fval2] = fminsearch(@lolik,xbeta0,options,d10,d20,d21,d22,A);
            end
        end
        % A Nelder-Mead (1965) Simplex method for MLE
        
        if (fval2 <= fval1)
            beta = exp(xbeta2); % ML Estimate of Beta
        else
            beta = exp(xbeta1); % ML Estimate of Beta
        end
        
        iden = zeros(4,1);
        
        if ((isnan(beta) == iden)&(isinf(beta) == iden))
        
            tbeta = [tbeta;beta'];
            
        end
    end 
end

    
output = std(tbeta);




